@@ -1,9 +1,9 @@
 /*
- * "$Id: snmp-supplies.c 10493 2012-05-21 22:33:19Z mike $"
+ * "$Id: snmp-supplies.c 11173 2013-07-23 12:31:34Z msweet $"
  *
  *   SNMP supplies functions for CUPS.
  *
- *   Copyright 2008-2012 by Apple Inc.
+ *   Copyright 2008-2013 by Apple Inc.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Apple Inc. and are protected by Federal copyright
@@ -163,7 +163,7 @@ static const int	prtMarkerSuppliesSupplyUnit[] =
 			 sizeof(prtMarkerSuppliesSupplyUnit[0]));
 					/* Offset to supply index */
 
-static const backend_state_t const printer_states[] =
+static const backend_state_t printer_states[] =
 			{
 			  /* { CUPS_TC_lowPaper, "media-low-report" }, */
 			  { CUPS_TC_noPaper | CUPS_TC_inputTrayEmpty, "media-empty-warning" },
@@ -180,7 +180,7 @@ static const backend_state_t const printer_states[] =
 			  { CUPS_TC_outputFull, "output-area-full-warning" }
 			};
 
-static const backend_state_t const supply_states[] =
+static const backend_state_t supply_states[] =
 			{
 			  { CUPS_DEVELOPER_LOW, "developer-low-report" },
 			  { CUPS_DEVELOPER_EMPTY, "developer-empty-warning" },
@@ -1073,5 +1073,5 @@ utf16_to_utf8(
 
 
 /*
- * End of "$Id: snmp-supplies.c 10493 2012-05-21 22:33:19Z mike $".
+ * End of "$Id: snmp-supplies.c 11173 2013-07-23 12:31:34Z msweet $".
  */
