@@ -1,9 +1,9 @@
 /*
- * "$Id: lpq.c 8929 2009-12-15 22:40:37Z mike $"
+ * "$Id: lpq.c 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   "lpq" command for the Common UNIX Printing System (CUPS).
+ *   "lpq" command for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2013 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -28,12 +28,7 @@
  * Include necessary headers...
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <cups/string.h>
-#include <cups/cups.h>
-#include <cups/i18n.h>
-#include <cups/debug.h>
+#include <cups/cups-private.h>
 
 
 /*
@@ -44,7 +39,7 @@ static http_t	*connect_server(const char *, http_t *);
 static int	show_jobs(const char *, http_t *, const char *,
 		          const char *, const int, const int);
 static void	show_printer(const char *, http_t *, const char *);
-static void	usage(void);
+static void	usage(void) __attribute__((noreturn));
 
 
 /*
@@ -65,8 +60,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		all,			/* All printers */
 		interval,		/* Reporting interval */
 		longstatus;		/* Show file details */
-  int		num_dests;		/* Number of destinations */
-  cups_dest_t	*dests;			/* Destinations */
+  cups_dest_t	*named_dest;		/* Named destination */
 
 
   _cupsSetLocale(argv);
@@ -82,8 +76,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   interval   = 0;
   longstatus = 0;
   all        = 0;
-  num_dests  = 0;
-  dests      = NULL;
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '+')
@@ -99,8 +91,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    if (http)
 	      httpEncryption(http, HTTP_ENCRYPT_REQUIRED);
 #else
-            _cupsLangPrintf(stderr,
-	                    _("%s: Sorry, no encryption support compiled in!\n"),
+            _cupsLangPrintf(stderr, _("%s: Sorry, no encryption support."),
 	                    argv[0]);
 #endif /* HAVE_SSL */
 	    break;
@@ -115,15 +106,14 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		                _("%s: Error - expected username after "
-				  "\'-U\' option!\n"),
-		        	argv[0]);
+				  "\"-U\" option."), argv[0]);
 	        return (1);
 	      }
 
               cupsSetUser(argv[i]);
 	    }
 	    break;
-	    
+
         case 'P' : /* Printer */
 	    if (argv[i][2])
 	      dest = argv[i] + 2;
@@ -134,8 +124,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 		httpClose(http);
-		cupsFreeDests(num_dests, dests);
-	        
+
 	        usage();
 	      }
 
@@ -147,22 +136,25 @@ main(int  argc,				/* I - Number of command-line arguments */
 
             http = connect_server(argv[0], http);
 
-            if (num_dests == 0)
-              num_dests = cupsGetDests2(http, &dests);
-
-            if (cupsGetDest(dest, instance, num_dests, dests) == NULL)
+            if ((named_dest = cupsGetNamedDest(http, dest, instance)) == NULL)
 	    {
-	      if (instance)
+	      if (cupsLastError() == IPP_BAD_REQUEST ||
+		  cupsLastError() == IPP_VERSION_NOT_SUPPORTED)
 		_cupsLangPrintf(stderr,
-		                _("%s: Error - unknown destination \"%s/%s\"!\n"),
+				_("%s: Error - add '/version=1.1' to server "
+				  "name."), argv[0]);
+	      else if (instance)
+		_cupsLangPrintf(stderr,
+		                _("%s: Error - unknown destination \"%s/%s\"."),
 		        	argv[0], dest, instance);
               else
-		_cupsLangPrintf(stderr,
-		                _("%s: Unknown destination \"%s\"!\n"),
+		_cupsLangPrintf(stderr, _("%s: Unknown destination \"%s\"."),
 				argv[0], dest);
 
 	      return (1);
 	    }
+
+	    cupsFreeDests(1, named_dest);
 	    break;
 
 	case 'a' : /* All printers */
@@ -186,8 +178,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      {
 	        _cupsLangPrintf(stderr,
 		        	_("%s: Error - expected hostname after "
-			          "\'-h\' option!\n"),
-				argv[0]);
+			          "\"-h\" option."), argv[0]);
 		return (1);
               }
 	      else
@@ -201,7 +192,6 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 	default :
 	    httpClose(http);
-	    cupsFreeDests(num_dests, dests);
 
 	    usage();
 	    break;
@@ -216,15 +206,17 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (dest == NULL && !all)
   {
-    if (num_dests == 0)
-      num_dests = cupsGetDests2(http, &dests);
-
-    for (i = 0; i < num_dests; i ++)
-      if (dests[i].is_default)
-	dest = dests[i].name;
-
-    if (dest == NULL)
+    if ((named_dest = cupsGetNamedDest(http, NULL, NULL)) == NULL)
     {
+      if (cupsLastError() == IPP_BAD_REQUEST ||
+          cupsLastError() == IPP_VERSION_NOT_SUPPORTED)
+      {
+	_cupsLangPrintf(stderr,
+	                _("%s: Error - add '/version=1.1' to server name."),
+			argv[0]);
+        return (1);
+      }
+
       val = NULL;
 
       if ((dest = getenv("LPDEST")) == NULL)
@@ -240,19 +232,20 @@ main(int  argc,				/* I - Number of command-line arguments */
       else
 	val = "LPDEST";
 
-      if (dest && !cupsGetDest(dest, NULL, num_dests, dests))
+      if (dest && val)
 	_cupsLangPrintf(stderr,
-	                _("%s: error - %s environment variable names "
-			  "non-existent destination \"%s\"!\n"),
-        	        argv[0], val, dest);
+	                _("%s: Error - %s environment variable names "
+			  "non-existent destination \"%s\"."), argv[0], val,
+			dest);
       else
 	_cupsLangPrintf(stderr,
-	                _("%s: error - no default destination available.\n"),
+	                _("%s: Error - no default destination available."),
 			argv[0]);
       httpClose(http);
-      cupsFreeDests(num_dests, dests);
       return (1);
     }
+
+    dest = named_dest->name;
   }
 
  /*
@@ -279,7 +272,6 @@ main(int  argc,				/* I - Number of command-line arguments */
   * Close the connection to the server and return...
   */
 
-  cupsFreeDests(num_dests, dests);
   httpClose(http);
 
   return (0);
@@ -301,7 +293,7 @@ connect_server(const char *command,	/* I - Command name */
 
     if (http == NULL)
     {
-      _cupsLangPrintf(stderr, _("%s: Unable to connect to server\n"), command);
+      _cupsLangPrintf(stderr, _("%s: Unable to connect to server."), command);
       exit(1);
     }
   }
@@ -380,6 +372,7 @@ show_jobs(const char *command,		/* I - Command name */
   *    attributes-natural-language
   *    job-uri or printer-uri
   *    requested-attributes
+  *    requesting-user-name
   */
 
   request = ippNewRequest(id ? IPP_GET_JOB_ATTRIBUTES : IPP_GET_JOBS);
@@ -408,6 +401,9 @@ show_jobs(const char *command,		/* I - Command name */
                  "requesting-user-name", NULL, user);
     ippAddBoolean(request, IPP_TAG_OPERATION, "my-jobs", 1);
   }
+  else
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                 "requesting-user-name", NULL, cupsUser());
 
   ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
                 "requested-attributes",
@@ -423,7 +419,7 @@ show_jobs(const char *command,		/* I - Command name */
   {
     if (response->request.status.status_code > IPP_OK_CONFLICT)
     {
-      _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+      _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
       ippDelete(response);
       return (0);
     }
@@ -456,8 +452,8 @@ show_jobs(const char *command,		/* I - Command name */
       jobpriority = 50;
 #endif /* __osf__ */
       jobstate    = IPP_JOB_PENDING;
-      jobname     = "untitled";
-      jobuser     = NULL;
+      jobname     = "unknown";
+      jobuser     = "unknown";
       jobdest     = NULL;
       jobcopies   = 1;
 
@@ -516,12 +512,13 @@ show_jobs(const char *command,		/* I - Command name */
       if (!longstatus && jobcount == 0)
 #ifdef __osf__
 	_cupsLangPuts(stdout,
+	              /* TRANSLATORS: Pri is job priority. */
 	              _("Rank   Owner      Pri  Job        Files"
-		        "                       Total Size\n"));
+		        "                       Total Size"));
 #else
 	_cupsLangPuts(stdout,
 	              _("Rank    Owner   Job     File(s)"
-		        "                         Total Size\n"));
+		        "                         Total Size"));
 #endif /* __osf__ */
 
       jobcount ++;
@@ -557,20 +554,20 @@ show_jobs(const char *command,		/* I - Command name */
 	else
 	  strlcpy(namestr, jobname, sizeof(namestr));
 
-        _cupsLangPrintf(stdout, _("%s: %-33.33s [job %d localhost]\n"),
+        _cupsLangPrintf(stdout, _("%s: %-33.33s [job %d localhost]"),
 	                jobuser, rankstr, jobid);
-        _cupsLangPrintf(stdout, _("        %-39.39s %.0f bytes\n"),
+        _cupsLangPrintf(stdout, _("        %-39.39s %.0f bytes"),
 	                namestr, 1024.0 * jobsize);
       }
       else
 #ifdef __osf__
         _cupsLangPrintf(stdout,
-	                _("%-6s %-10.10s %-4d %-10d %-27.27s %.0f bytes\n"),
+	                _("%-6s %-10.10s %-4d %-10d %-27.27s %.0f bytes"),
 			rankstr, jobuser, jobpriority, jobid, jobname,
 			1024.0 * jobsize);
 #else
         _cupsLangPrintf(stdout,
-	                _("%-7s %-7.7s %-7d %-31.31s %.0f bytes\n"),
+	                _("%-7s %-7.7s %-7d %-31.31s %.0f bytes"),
 			rankstr, jobuser, jobid, jobname, 1024.0 * jobsize);
 #endif /* __osf */
 
@@ -582,12 +579,12 @@ show_jobs(const char *command,		/* I - Command name */
   }
   else
   {
-    _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+    _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
     return (0);
   }
 
   if (jobcount == 0)
-    _cupsLangPuts(stdout, _("no entries\n"));
+    _cupsLangPuts(stdout, _("no entries"));
 
   return (jobcount);
 }
@@ -636,7 +633,7 @@ show_printer(const char *command,	/* I - Command name */
   {
     if (response->request.status.status_code > IPP_OK_CONFLICT)
     {
-      _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+      _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
       ippDelete(response);
       return;
     }
@@ -649,21 +646,21 @@ show_printer(const char *command,	/* I - Command name */
     switch (state)
     {
       case IPP_PRINTER_IDLE :
-          _cupsLangPrintf(stdout, _("%s is ready\n"), dest);
+          _cupsLangPrintf(stdout, _("%s is ready"), dest);
 	  break;
       case IPP_PRINTER_PROCESSING :
-          _cupsLangPrintf(stdout, _("%s is ready and printing\n"),
+          _cupsLangPrintf(stdout, _("%s is ready and printing"),
 	                  dest);
 	  break;
       case IPP_PRINTER_STOPPED :
-          _cupsLangPrintf(stdout, _("%s is not ready\n"), dest);
+          _cupsLangPrintf(stdout, _("%s is not ready"), dest);
 	  break;
     }
 
     ippDelete(response);
   }
   else
-    _cupsLangPrintf(stderr, "%s: %s\n", command, cupsLastErrorString());
+    _cupsLangPrintf(stderr, "%s: %s", command, cupsLastErrorString());
 }
 
 
@@ -676,11 +673,11 @@ usage(void)
 {
   _cupsLangPuts(stderr,
                 _("Usage: lpq [-P dest] [-U username] [-h hostname[:port]] "
-		  "[-l] [+interval]\n"));
+		  "[-l] [+interval]"));
   exit(1);
 }
 
 
 /*
- * End of "$Id: lpq.c 8929 2009-12-15 22:40:37Z mike $".
+ * End of "$Id: lpq.c 11173 2013-07-23 12:31:34Z msweet $".
  */
