@@ -1,9 +1,9 @@
 /*
- * "$Id: cgi-private.h 7940 2008-09-16 00:45:16Z mike $"
+ * "$Id: cgi-private.h 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   Private CGI definitions for the Common UNIX Printing System (CUPS).
+ *   Private CGI definitions for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -18,9 +18,10 @@
  */
 
 #include "cgi.h"
-#include <cups/debug.h>
-#include <cups/i18n.h>
-#include <cups/string.h>
+#include <cups/debug-private.h>
+#include <cups/language-private.h>
+#include <cups/string-private.h>
+#include <cups/ipp-private.h>	/* TODO: Update so we don't need this */
 
 
 /*
@@ -31,5 +32,5 @@
 
 
 /*
- * End of "$Id: cgi-private.h 7940 2008-09-16 00:45:16Z mike $".
+ * End of "$Id: cgi-private.h 11173 2013-07-23 12:31:34Z msweet $".
  */
