@@ -1,9 +1,9 @@
 /*
- * "$Id: template.c 8980 2010-02-08 17:33:31Z mike $"
+ * "$Id: template.c 11173 2013-07-23 12:31:34Z msweet $"
  *
  *   CGI template function.
  *
- *   Copyright 2007-2010 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -369,6 +369,20 @@ cgi_copy(FILE *out,			/* I - Output file */
 
         continue;
       }
+      else if (name[0] == '$')
+      {
+       /*
+        * Insert cookie value or nothing if not defined.
+	*/
+
+        if ((value = cgiGetCookie(name + 1)) != NULL)
+	  outptr = value;
+	else
+	{
+	  outval[0] = '\0';
+	  outptr    = outval;
+	}
+      }
       else
       {
        /*
@@ -409,7 +423,7 @@ cgi_copy(FILE *out,			/* I - Output file */
 	{
 	  if (uriencode)
 	    cgi_puturi(outptr, out);
-	  else if (!strcasecmp(name, "?cupsdconf_default"))
+	  else if (!_cups_strcasecmp(name, "?cupsdconf_default"))
 	    fputs(outptr, stdout);
 	  else
 	    cgi_puts(outptr, out);
@@ -522,16 +536,16 @@ cgi_copy(FILE *out,			/* I - Output file */
         switch (op)
 	{
 	  case '<' :
-	      result = strcasecmp(outptr, compare) < 0;
+	      result = _cups_strcasecmp(outptr, compare) < 0;
 	      break;
 	  case '>' :
-	      result = strcasecmp(outptr, compare) > 0;
+	      result = _cups_strcasecmp(outptr, compare) > 0;
 	      break;
 	  case '=' :
-	      result = strcasecmp(outptr, compare) == 0;
+	      result = _cups_strcasecmp(outptr, compare) == 0;
 	      break;
 	  case '!' :
-	      result = strcasecmp(outptr, compare) != 0;
+	      result = _cups_strcasecmp(outptr, compare) != 0;
 	      break;
 	  case '~' :
 	      fprintf(stderr, "DEBUG: Regular expression \"%s\"\n", compare);
@@ -650,7 +664,7 @@ cgi_puts(const char *s,			/* I - String to output */
       * Pass <A HREF="url"> and </A>, otherwise quote it...
       */
 
-      if (!strncasecmp(s, "<A HREF=\"", 9))
+      if (!_cups_strncasecmp(s, "<A HREF=\"", 9))
       {
         fputs("<A HREF=\"", out);
 	s += 9;
@@ -670,7 +684,7 @@ cgi_puts(const char *s,			/* I - String to output */
 
 	fputs("\">", out);
       }
-      else if (!strncasecmp(s, "</A>", 4))
+      else if (!_cups_strncasecmp(s, "</A>", 4))
       {
         fputs("</A>", out);
 	s += 3;
@@ -715,5 +729,5 @@ cgi_puturi(const char *s,		/* I - String to output */
 
 
 /*
- * End of "$Id: template.c 8980 2010-02-08 17:33:31Z mike $".
+ * End of "$Id: template.c 11173 2013-07-23 12:31:34Z msweet $".
  */
