@@ -1,11 +1,11 @@
 /*
- * "$Id$"
+ * "$Id: dir.h 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   Public directory definitions for the Common UNIX Printing System (CUPS).
+ *   Public directory definitions for CUPS.
  *
  *   This set of APIs abstracts enumeration of directory entries.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -23,6 +23,7 @@
  * Include necessary headers...
  */
 
+#  include "versioning.h"
 #  include <sys/stat.h>
 
 
@@ -52,10 +53,10 @@ typedef struct cups_dentry_s		/**** Directory entry type ****/
  * Prototypes...
  */
 
-extern void		cupsDirClose(cups_dir_t *dp);
-extern cups_dir_t	*cupsDirOpen(const char *directory);
-extern cups_dentry_t	*cupsDirRead(cups_dir_t *dp);
-extern void		cupsDirRewind(cups_dir_t *dp);
+extern void		cupsDirClose(cups_dir_t *dp) _CUPS_API_1_2;
+extern cups_dir_t	*cupsDirOpen(const char *directory) _CUPS_API_1_2;
+extern cups_dentry_t	*cupsDirRead(cups_dir_t *dp) _CUPS_API_1_2;
+extern void		cupsDirRewind(cups_dir_t *dp) _CUPS_API_1_2;
 
 
 #  ifdef __cplusplus
@@ -64,5 +65,5 @@ extern void		cupsDirRewind(cups_dir_t *dp);
 #endif /* !_CUPS_DIR_H_ */
 
 /*
- * End of "$Id$".
+ * End of "$Id: dir.h 11173 2013-07-23 12:31:34Z msweet $".
  */
