@@ -1,32 +1,32 @@
 /*
-  Copyright 2005 by Easy Software Products
-
-  This source file implements private APIs and should not be used in
-  CUPS-based applications.
-
-  Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
-
-  This software is provided 'as-is', without any express or implied
-  warranty.  In no event will the authors be held liable for any damages
-  arising from the use of this software.
-
-  Permission is granted to anyone to use this software for any purpose,
-  including commercial applications, and to alter it and redistribute it
-  freely, subject to the following restrictions:
-
-  1. The origin of this software must not be misrepresented; you must not
-     claim that you wrote the original software. If you use this software
-     in a product, an acknowledgment in the product documentation would be
-     appreciated but is not required.
-  2. Altered source versions must be plainly marked as such, and must not be
-     misrepresented as being the original software.
-  3. This notice may not be removed or altered from any source distribution.
-
-  L. Peter Deutsch
-  ghost@aladdin.com
-
+ * "$Id: md5.c 11173 2013-07-23 12:31:34Z msweet $"
+ *
+ *   Private MD5 implementation for CUPS.
+ *
+ *   Copyright 2007-2010 by Apple Inc.
+ *   Copyright 2005 by Easy Software Products
+ *
+ *   Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+ *
+ *   This software is provided 'as-is', without any express or implied
+ *   warranty.  In no event will the authors be held liable for any damages
+ *   arising from the use of this software.
+ *
+ *   Permission is granted to anyone to use this software for any purpose,
+ *   including commercial applications, and to alter it and redistribute it
+ *   freely, subject to the following restrictions:
+ *
+ *   1. The origin of this software must not be misrepresented; you must not
+ *      claim that you wrote the original software. If you use this software
+ *      in a product, an acknowledgment in the product documentation would be
+ *      appreciated but is not required.
+ *   2. Altered source versions must be plainly marked as such, and must not be
+ *      misrepresented as being the original software.
+ *   3. This notice may not be removed or altered from any source distribution.
+ *
+ *   L. Peter Deutsch
+ *   ghost@aladdin.com
  */
-/*$Id: md5.c 5232 2006-03-05 17:59:19Z mike $ */
 /*
   Independent implementation of MD5 (RFC 1321).
 
@@ -43,11 +43,8 @@
   1999-05-03 lpd Original version.
  */
 
-#include "md5.h"
-#include "string.h"
-#ifdef __APPLE__
-#  include "md5-apple.h"
-#endif /* __APPLE__ */
+#include "md5-private.h"
+#include "string-private.h"
 
 #define T1 0xd76aa478
 #define T2 0xe8c7b756
@@ -342,3 +339,8 @@ _cupsMD5Finish(_cups_md5_state_t *pms, unsigned char digest[16])
     for (i = 0; i < 16; ++i)
 	digest[i] = (unsigned char)(pms->abcd[i >> 2] >> ((i & 3) << 3));
 }
+
+
+/*
+ * End of "$Id: md5.c 11173 2013-07-23 12:31:34Z msweet $".
+ */
