@@ -1,9 +1,9 @@
 /*
- * "$Id: snmp.c 8819 2009-09-22 18:47:36Z mike $"
+ * "$Id: snmp.c 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   SNMP functions for the Common UNIX Printing System (CUPS).
+ *   SNMP functions for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 2006-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -58,12 +58,10 @@
  * Include necessary headers.
  */
 
-#include "globals.h"
-#include "debug.h"
+#include "cups-private.h"
 #include "snmp-private.h"
-#include <errno.h>
 #ifdef HAVE_POLL
-#  include <sys/poll.h>
+#  include <poll.h>
 #endif /* HAVE_POLL */
 
 
@@ -89,14 +87,14 @@ static char		*asn1_get_string(unsigned char **buffer,
 			                 unsigned char *bufend,
 			                 int length, char *string,
 			                 int strsize);
-static int		asn1_get_length(unsigned char **buffer,
+static unsigned		asn1_get_length(unsigned char **buffer,
 			                unsigned char *bufend);
 static int		asn1_get_type(unsigned char **buffer,
 			              unsigned char *bufend);
 static void		asn1_set_integer(unsigned char **buffer,
 			                 int integer);
 static void		asn1_set_length(unsigned char **buffer,
-			                int length);
+			                unsigned length);
 static void		asn1_set_oid(unsigned char **buffer,
 			             const int *oid);
 static void		asn1_set_packed(unsigned char **buffer,
@@ -180,7 +178,7 @@ _cupsSNMPDefaultCommunity(void)
     {
       linenum = 0;
       while (cupsFileGetConf(fp, line, sizeof(line), &value, &linenum))
-	if (!strcasecmp(line, "Community") && value)
+	if (!_cups_strcasecmp(line, "Community") && value)
 	{
 	  strlcpy(cg->snmp_community, value, sizeof(cg->snmp_community));
 	  break;
@@ -730,7 +728,7 @@ _cupsSNMPWrite(
   packet.request_type = request_type;
   packet.request_id   = request_id;
   packet.object_type  = CUPS_ASN1_NULL_VALUE;
-  
+
   strlcpy(packet.community, community, sizeof(packet.community));
 
   for (i = 0; oid[i] >= 0 && i < (CUPS_SNMP_MAX_OID - 1); i ++)
@@ -763,12 +761,7 @@ _cupsSNMPWrite(
 
   temp = *address;
 
-#ifdef AF_INET6
-  if (temp.addr.sa_family == AF_INET6)
-    temp.ipv6.sin6_port = htons(CUPS_SNMP_PORT);
-  else
-#endif /* AF_INET6 */
-  temp.ipv4.sin_port = htons(CUPS_SNMP_PORT);
+  _httpAddrSetPort(&temp, CUPS_SNMP_PORT);
 
   return (sendto(fd, buffer, bytes, 0, (void *)&temp,
                  httpAddrLength(&temp)) == bytes);
@@ -970,7 +963,7 @@ asn1_debug(const char    *prefix,	/* I - Prefix string */
     }
   }
 }
-          
+
 
 /*
  * 'asn1_decode_snmp()' - Decode a SNMP packet.
@@ -1092,7 +1085,7 @@ asn1_decode_snmp(unsigned char *buffer,	/* I - Buffer */
 		    packet->object_value.string.num_bytes = length;
 		    asn1_get_string(&bufptr, bufend, length,
 		                    (char *)packet->object_value.string.bytes,
-				    CUPS_SNMP_MAX_STRING);
+				    sizeof(packet->object_value.string.bytes));
 	            break;
 
 	        case CUPS_ASN1_OID :
@@ -1286,6 +1279,12 @@ asn1_get_integer(
   int	value;				/* Integer value */
 
 
+  if (length > sizeof(int))
+  {
+    (*buffer) += length;
+    return (0);
+  }
+
   for (value = (**buffer & 0x80) ? -1 : 0;
        length > 0 && *buffer < bufend;
        length --, (*buffer) ++)
@@ -1299,18 +1298,32 @@ asn1_get_integer(
  * 'asn1_get_length()' - Get a value length.
  */
 
-static int				/* O  - Length */
+static unsigned				/* O  - Length */
 asn1_get_length(unsigned char **buffer,	/* IO - Pointer in buffer */
 		unsigned char *bufend)	/* I  - End of buffer */
 {
-  int	length;				/* Length */
+  unsigned	length;			/* Length */
 
 
   length = **buffer;
   (*buffer) ++;
 
   if (length & 128)
-    length = asn1_get_integer(buffer, bufend, length & 127);
+  {
+    int	count;				/* Number of bytes for length */
+
+
+    if ((count = length & 127) > sizeof(unsigned))
+    {
+      (*buffer) += count;
+      return (0);
+    }
+
+    for (length = 0;
+	 count > 0 && *buffer < bufend;
+	 count --, (*buffer) ++)
+      length = (length << 8) | **buffer;
+  }
 
   return (length);
 }
@@ -1412,6 +1425,9 @@ asn1_get_string(
     char          *string,		/* I  - String buffer */
     int           strsize)		/* I  - String buffer size */
 {
+  if (length > (bufend - *buffer))
+    length = bufend - *buffer;
+
   if (length < 0)
   {
    /*
@@ -1529,7 +1545,7 @@ asn1_set_integer(unsigned char **buffer,/* IO - Pointer in buffer */
 
 static void
 asn1_set_length(unsigned char **buffer,	/* IO - Pointer in buffer */
-		int           length)	/* I  - Length value */
+		unsigned      length)	/* I  - Length value */
 {
   if (length > 255)
   {
@@ -1717,5 +1733,5 @@ snmp_set_error(cups_snmp_t *packet,	/* I - Packet */
 
 
 /*
- * End of "$Id: snmp.c 8819 2009-09-22 18:47:36Z mike $".
+ * End of "$Id: snmp.c 11173 2013-07-23 12:31:34Z msweet $".
  */
