@@ -1,9 +1,9 @@
 /*
- * "$Id: transcode.h 6649 2007-07-11 21:46:42Z mike $"
+ * "$Id: transcode.h 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   Transcoding definitions for the Common UNIX Printing System (CUPS).
+ *   Transcoding definitions for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -57,17 +57,17 @@ typedef unsigned long  cups_vbcs_t;	/* VBCS Legacy 32-bit unit */
 extern int	cupsCharsetToUTF8(cups_utf8_t *dest,
 				  const char *src,
 				  const int maxout,
-				  const cups_encoding_t encoding);
+				  const cups_encoding_t encoding) _CUPS_API_1_2;
 extern int	cupsUTF8ToCharset(char *dest,
 				  const cups_utf8_t *src,
 				  const int maxout,
-				  const cups_encoding_t encoding);
+				  const cups_encoding_t encoding) _CUPS_API_1_2;
 extern int	cupsUTF8ToUTF32(cups_utf32_t *dest,
 				const cups_utf8_t *src,
-				const int maxout);
+				const int maxout) _CUPS_API_1_2;
 extern int	cupsUTF32ToUTF8(cups_utf8_t *dest,
 				const cups_utf32_t *src,
-				const int maxout);
+				const int maxout) _CUPS_API_1_2;
 
 #  ifdef __cplusplus
 }
@@ -77,5 +77,5 @@ extern int	cupsUTF32ToUTF8(cups_utf8_t *dest,
 
 
 /*
- * End of "$Id: transcode.h 6649 2007-07-11 21:46:42Z mike $"
+ * End of "$Id: transcode.h 11173 2013-07-23 12:31:34Z msweet $"
  */
