@@ -1,10 +1,9 @@
 /*
- * "$Id: rastertoepson.c 7222 2008-01-16 22:20:33Z mike $"
+ * "$Id: rastertoepson.c 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   EPSON ESC/P and ESC/P2 filter for the Common UNIX Printing System
- *   (CUPS).
+ *   EPSON ESC/P and ESC/P2 filter for CUPS.
  *
- *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1993-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -32,10 +31,9 @@
 
 #include <cups/cups.h>
 #include <cups/ppd.h>
-#include <cups/string.h>
-#include <cups/i18n.h>
-#include "raster.h"
-#include <stdlib.h>
+#include <cups/string-private.h>
+#include <cups/language-private.h>
+#include <cups/raster.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -77,7 +75,8 @@ int		DotBit,			/* Bit in buffers */
 		LineCount,		/* # of lines processed */
 		EvenOffset,		/* Offset into 'even' buffers */
 		OddOffset,		/* Offset into 'odd' buffers */
-		Shingling;		/* Shingle output? */
+		Shingling,		/* Shingle output? */
+		Canceled;		/* Has the current job been canceled? */
 
 
 /*
@@ -85,15 +84,15 @@ int		DotBit,			/* Bit in buffers */
  */
 
 void	Setup(void);
-void	StartPage(const ppd_file_t *ppd, const cups_page_header_t *header);
-void	EndPage(const cups_page_header_t *header);
+void	StartPage(const ppd_file_t *ppd, const cups_page_header2_t *header);
+void	EndPage(const cups_page_header2_t *header);
 void	Shutdown(void);
 
 void	CancelJob(int sig);
 void	CompressData(const unsigned char *line, int length, int plane,
 	             int type, int xstep, int ystep);
-void	OutputLine(const cups_page_header_t *header);
-void	OutputRows(const cups_page_header_t *header, int row);
+void	OutputLine(const cups_page_header2_t *header);
+void	OutputRows(const cups_page_header2_t *header, int row);
 
 
 /*
@@ -103,7 +102,7 @@ void	OutputRows(const cups_page_header_t *header, int row);
 void
 Setup(void)
 {
-  const char	*device_uri;	/* The device for the printer... */
+  const char	*device_uri;		/* The device for the printer... */
 
 
  /*
@@ -122,33 +121,14 @@ Setup(void)
  */
 
 void
-StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
-          const cups_page_header_t *header)	/* I - Page header */
+StartPage(
+    const ppd_file_t         *ppd,	/* I - PPD file */
+    const cups_page_header2_t *header)	/* I - Page header */
 {
-  int	n, t;					/* Numbers */
-  int	plane;					/* Looping var */
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;			/* Actions for POSIX signals */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
+  int	n, t;				/* Numbers */
+  int	plane;				/* Looping var */
 
 
- /*
-  * Register a signal handler to eject the current page if the
-  * job is cancelled.
-  */
-
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGTERM, CancelJob);
-#elif defined(HAVE_SIGACTION)
-  memset(&action, 0, sizeof(action));
-
-  sigemptyset(&action.sa_mask);
-  action.sa_handler = CancelJob;
-  sigaction(SIGTERM, &action, NULL);
-#else
-  signal(SIGTERM, CancelJob);
-#endif /* HAVE_SIGSET */
-
  /*
   * Send a reset sequence.
   */
@@ -162,8 +142,6 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
   * See which type of printer we are using...
   */
 
-  EjectPage = header->Margins[0] || header->Margins[1];
-    
   switch (Model)
   {
     case EPSON_9PIN :
@@ -183,8 +161,8 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 
 	printf("\033l%c\033Q%c", 0,	/* Side margins */
                       (int)(10.0 * header->PageSize[0] / 72.0 + 0.5));
-	printf("\033C%c%c", 0,		/* Page length */
-                      (int)(header->PageSize[1] / 72.0 + 0.5));
+	printf("\033\062\033C%c",	/* Page length in 1/6th inches */
+		      (int)(header->PageSize[1] / 12.0 + 0.5));
 	printf("\033N%c", 0);		/* Bottom margin */
         printf("\033O");		/* No perforation skip */
 
@@ -298,7 +276,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
 
   if ((Planes[0] = malloc(header->cupsBytesPerLine)) == NULL)
   {
-    fputs("ERROR: Unable to allocate memory!\n", stderr);
+    fputs("ERROR: Unable to allocate memory\n", stderr);
     exit(1);
   }
 
@@ -309,7 +287,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
   {
     if ((CompBuffer = calloc(2, header->cupsWidth)) == NULL)
     {
-      fputs("ERROR: Unable to allocate memory!\n", stderr);
+      fputs("ERROR: Unable to allocate memory\n", stderr);
       exit(1);
     }
   }
@@ -321,7 +299,7 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
     if ((LineBuffers[0] = calloc(DotBytes,
                                  header->cupsWidth * (Shingling + 1))) == NULL)
     {
-      fputs("ERROR: Unable to allocate memory!\n", stderr);
+      fputs("ERROR: Unable to allocate memory\n", stderr);
       exit(1);
     }
 
@@ -339,13 +317,9 @@ StartPage(const ppd_file_t         *ppd,	/* I - PPD file */
  */
 
 void
-EndPage(const cups_page_header_t *header)	/* I - Page header */
+EndPage(
+    const cups_page_header2_t *header)	/* I - Page header */
 {
-#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;			/* Actions for POSIX signals */
-#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
-
-
   if (DotBytes && header)
   {
    /*
@@ -373,26 +347,9 @@ EndPage(const cups_page_header_t *header)	/* I - Page header */
   * Eject the current page...
   */
 
-  if (EjectPage)
-    putchar(12);		/* Form feed */
+  putchar(12);				/* Form feed */
   fflush(stdout);
 
- /*
-  * Unregister the signal handler...
-  */
-
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGTERM, SIG_IGN);
-#elif defined(HAVE_SIGACTION)
-  memset(&action, 0, sizeof(action));
-
-  sigemptyset(&action.sa_mask);
-  action.sa_handler = SIG_IGN;
-  sigaction(SIGTERM, &action, NULL);
-#else
-  signal(SIGTERM, SIG_IGN);
-#endif /* HAVE_SIGSET */
-
  /*
   * Free memory...
   */
@@ -429,31 +386,9 @@ Shutdown(void)
 void
 CancelJob(int sig)			/* I - Signal */
 {
-  int	i;				/* Looping var */
-
-
   (void)sig;
 
- /*
-  * Send out lots of NUL bytes to clear out any pending raster data...
-  */
-
-  if (DotBytes)
-    i = DotBytes * 360 * 8;
-  else
-    i = 720;
-
-  for (; i > 0; i --)
-    putchar(0);
-
- /*
-  * End the current page and exit...
-  */
-
-  EndPage(NULL);
-  Shutdown();
-
-  exit(0);
+  Canceled = 1;
 }
 
 
@@ -670,7 +605,8 @@ CompressData(const unsigned char *line,	/* I - Data to compress */
  */
 
 void
-OutputLine(const cups_page_header_t *header)	/* I - Page header */
+OutputLine(
+    const cups_page_header2_t *header)	/* I - Page header */
 {
   if (header->cupsRowCount)
   {
@@ -838,14 +774,15 @@ OutputLine(const cups_page_header_t *header)	/* I - Page header */
  */
 
 void
-OutputRows(const cups_page_header_t *header,	/* I - Page image header */
-           int                      row)	/* I - Row number (0 or 1) */
+OutputRows(
+    const cups_page_header2_t *header,	/* I - Page image header */
+    int                      row)	/* I - Row number (0 or 1) */
 {
-  unsigned	i, n;				/* Looping vars */
-  int		dot_count,			/* Number of bytes to print */
-                dot_min;			/* Minimum number of bytes */
-  unsigned char *dot_ptr,			/* Pointer to print data */
-		*ptr;				/* Current data */
+  unsigned	i, n;			/* Looping vars */
+  int		dot_count,		/* Number of bytes to print */
+                dot_min;		/* Minimum number of bytes */
+  unsigned char *dot_ptr,		/* Pointer to print data */
+		*ptr;			/* Current data */
 
 
   dot_min = DotBytes * DotColumns;
@@ -1012,16 +949,19 @@ OutputRows(const cups_page_header_t *header,	/* I - Page image header */
  * 'main()' - Main entry and processing of driver.
  */
 
-int				/* O - Exit status */
-main(int  argc,			/* I - Number of command-line arguments */
-     char *argv[])		/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  int			fd;	/* File descriptor */
-  cups_raster_t		*ras;	/* Raster stream for printing */
-  cups_page_header_t	header;	/* Page header from file */
-  ppd_file_t		*ppd;	/* PPD file */
-  int			page;	/* Current page */
-  int			y;	/* Current line */
+  int			fd;		/* File descriptor */
+  cups_raster_t		*ras;		/* Raster stream for printing */
+  cups_page_header2_t	header;		/* Page header from file */
+  ppd_file_t		*ppd;		/* PPD file */
+  int			page;		/* Current page */
+  int			y;		/* Current line */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;		/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
@@ -1041,8 +981,9 @@ main(int  argc,			/* I - Number of command-line arguments */
     * and return.
     */
 
-    fprintf(stderr, _("Usage: %s job-id user title copies options [file]\n"),
-            argv[0]);
+    _cupsLangPrintFilter(stderr, "ERROR",
+                         _("%s job-id user title copies options [file]"),
+                         "rastertoepson");
     return (1);
   }
 
@@ -1054,7 +995,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   {
     if ((fd = open(argv[6], O_RDONLY)) == -1)
     {
-      perror("ERROR: Unable to open raster file - ");
+      _cupsLangPrintError("ERROR", _("Unable to open raster file"));
       sleep(1);
       return (1);
     }
@@ -1064,13 +1005,46 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   ras = cupsRasterOpen(fd, CUPS_RASTER_READ);
 
+ /*
+  * Register a signal handler to eject the current page if the
+  * job is cancelled.
+  */
+
+  Canceled = 0;
+
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+  sigset(SIGTERM, CancelJob);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+
+  sigemptyset(&action.sa_mask);
+  action.sa_handler = CancelJob;
+  sigaction(SIGTERM, &action, NULL);
+#else
+  signal(SIGTERM, CancelJob);
+#endif /* HAVE_SIGSET */
+
  /*
   * Initialize the print device...
   */
 
   ppd = ppdOpenFile(getenv("PPD"));
-  if (ppd)
-    Model = ppd->model_number;
+  if (!ppd)
+  {
+    ppd_status_t	status;		/* PPD error */
+    int			linenum;	/* Line number */
+
+    _cupsLangPrintFilter(stderr, "ERROR",
+                         _("The PPD file could not be opened."));
+
+    status = ppdLastError(&linenum);
+
+    fprintf(stderr, "DEBUG: %s on line %d.\n", ppdErrorString(status), linenum);
+
+    return (1);
+  }
+
+  Model = ppd->model_number;
 
   Setup();
 
@@ -1080,15 +1054,19 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   page = 0;
 
-  while (cupsRasterReadHeader(ras, &header))
+  while (cupsRasterReadHeader2(ras, &header))
   {
    /*
     * Write a status message with the page number and number of copies.
     */
 
+    if (Canceled)
+      break;
+
     page ++;
 
     fprintf(stderr, "PAGE: %d %d\n", page, header.NumCopies);
+    _cupsLangPrintFilter(stderr, "INFO", _("Starting page %d."), page);
 
    /*
     * Start the page...
@@ -1106,9 +1084,17 @@ main(int  argc,			/* I - Number of command-line arguments */
       * Let the user know how far we have progressed...
       */
 
+      if (Canceled)
+	break;
+
       if ((y & 127) == 0)
-        fprintf(stderr, _("INFO: Printing page %d, %d%% complete...\n"), page,
-	        100 * y / header.cupsHeight);
+      {
+        _cupsLangPrintFilter(stderr, "INFO",
+	                     _("Printing page %d, %d%% complete."),
+			     page, 100 * y / header.cupsHeight);
+        fprintf(stderr, "ATTR: job-media-progress=%d\n",
+		100 * y / header.cupsHeight);
+      }
 
      /*
       * Read a line of graphics...
@@ -1128,7 +1114,12 @@ main(int  argc,			/* I - Number of command-line arguments */
     * Eject the page...
     */
 
+    _cupsLangPrintFilter(stderr, "INFO", _("Finished page %d."), page);
+
     EndPage(&header);
+
+    if (Canceled)
+      break;
   }
 
  /*
@@ -1152,14 +1143,15 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   if (page == 0)
-    fputs(_("ERROR: No pages found!\n"), stderr);
+  {
+    _cupsLangPrintFilter(stderr, "ERROR", _("No pages were found."));
+    return (1);
+  }
   else
-    fputs(_("INFO: Ready to print.\n"), stderr);
-
-  return (page == 0);
+    return (0);
 }
 
 
 /*
- * End of "$Id: rastertoepson.c 7222 2008-01-16 22:20:33Z mike $".
+ * End of "$Id: rastertoepson.c 11173 2013-07-23 12:31:34Z msweet $".
  */
