@@ -1,9 +1,9 @@
 /*
- * "$Id: conf.h 9120 2010-04-23 18:56:34Z mike $"
+ * "$Id: conf.h 11221 2013-08-06 16:16:01Z msweet $"
  *
- *   Configuration file definitions for CUPS.
+ *   Configuration file definitions for the CUPS scheduler.
  *
- *   Copyright 2007-2010 by Apple Inc.
+ *   Copyright 2007-2013 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -69,7 +69,7 @@ typedef enum
 
 #define PRINTCAP_BSD		0	/* Berkeley LPD format */
 #define PRINTCAP_SOLARIS	1	/* Solaris lpsched format */
-#define PRINTCAP_PLIST		2	/* Mac OS X plist format */
+#define PRINTCAP_PLIST		2	/* OS X plist format */
 
 
 /*
@@ -96,7 +96,9 @@ typedef struct
  */
 
 VAR char		*ConfigurationFile	VALUE(NULL),
-					/* Configuration file to use */
+					/* cupsd.conf file to use */
+			*CupsFilesFile		VALUE(NULL),
+					/* cups-files.conf file to use */
 			*ServerName		VALUE(NULL),
 					/* FQDN for server */
 			*ServerAdmin		VALUE(NULL),
@@ -169,6 +171,10 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Amount of automatic debug history */
 			FatalErrors		VALUE(CUPSD_FATAL_CONFIG),
 					/* Which errors are fatal? */
+			StrictConformance	VALUE(FALSE),
+					/* Require strict IPP conformance? */
+			SyncOnClose		VALUE(FALSE),
+					/* Call fsync() when closing files? */
 			LogFilePerm		VALUE(0644);
 					/* Permissions for log files */
 VAR cupsd_loglevel_t	LogLevel		VALUE(CUPSD_LOG_WARN);
@@ -183,8 +189,6 @@ VAR int			MaxClients		VALUE(100),
 					/* Maximum number of copies per job */
 			MaxLogSize		VALUE(1024 * 1024),
 					/* Maximum size of log files */
-			MaxPrinterHistory	VALUE(10),
-					/* Maximum printer state history */
 			MaxRequestSize		VALUE(0),
 					/* Maximum size of IPP requests */
 			HostNameLookups		VALUE(FALSE),
@@ -195,12 +199,6 @@ VAR int			MaxClients		VALUE(100),
 					/* Support the Keep-Alive option? */
 			KeepAliveTimeout	VALUE(DEFAULT_KEEPALIVE),
 					/* Timeout between requests */
-			ImplicitClasses		VALUE(TRUE),
-					/* Are classes implicitly created? */
-			ImplicitAnyClasses	VALUE(FALSE),
-					/* Create AnyPrinter classes? */
-			HideImplicitMembers	VALUE(TRUE),
-					/* Hide implicit class members? */
 			FileDevice		VALUE(FALSE),
 					/* Allow file: devices? */
 			FilterLimit		VALUE(0),
@@ -219,8 +217,10 @@ VAR int			MaxClients		VALUE(100),
 					/* Format of printcap file? */
 			DefaultShared		VALUE(TRUE),
 					/* Share printers by default? */
-			MultipleOperationTimeout VALUE(DEFAULT_TIMEOUT);
+			MultipleOperationTimeout VALUE(DEFAULT_TIMEOUT),
 					/* multiple-operation-time-out value */
+			WebInterface		VALUE(CUPS_DEFAULT_WEBIF);
+					/* Enable the web interface? */
 VAR cups_file_t		*AccessFile		VALUE(NULL),
 					/* Access log file */
 			*ErrorFile		VALUE(NULL),
@@ -248,47 +248,49 @@ VAR int			SSLOptions		VALUE(CUPSD_SSL_NONE);
 #endif /* HAVE_SSL */
 
 #ifdef HAVE_LAUNCHD
-VAR int			LaunchdTimeout		VALUE(DEFAULT_KEEPALIVE);
+VAR int			LaunchdTimeout		VALUE(10);
 					/* Time after which an idle cupsd will exit */
 #endif /* HAVE_LAUNCHD */
 
-#ifdef __APPLE__
-VAR int			AppleQuotas		VALUE(TRUE);
-					/* Use Apple PrintService Quotas instead of CUPS quotas */
-#endif  /* __APPLE__ */
-
 #ifdef HAVE_AUTHORIZATION_H
 VAR char		*SystemGroupAuthKey	VALUE(NULL);
 					/* System group auth key */
 #endif /* HAVE_AUTHORIZATION_H */
 
+#ifdef HAVE_GSSAPI
+VAR char		*GSSServiceName		VALUE(NULL);
+					/* GSS service name */
+int			HaveServerCreds		VALUE(0);
+					/* Do we have server credentials? */
+gss_cred_id_t		ServerCreds;	/* Server's GSS credentials */
+#endif /* HAVE_GSSAPI */
+
 
 /*
  * Prototypes...
  */
 
 extern void	cupsdAddAlias(cups_array_t *aliases, const char *name);
+extern int	cupsdCheckLogFile(cups_file_t **lf, const char *logname);
 extern int	cupsdCheckPermissions(const char *filename,
 		                      const char *suffix, int mode,
 	 			      int user, int group, int is_dir,
 				      int create_dir);
+extern int	cupsdCheckProgram(const char *filename, cupsd_printer_t *p);
+extern int	cupsdDefaultAuthType(void);
 extern void	cupsdFreeAliases(cups_array_t *aliases);
 extern char	*cupsdGetDateTime(struct timeval *t, cupsd_time_t format);
+extern void	cupsdLogFCMessage(void *context, _cups_fc_result_t result,
+		                  const char *message);
 #ifdef HAVE_GSSAPI
 extern int	cupsdLogGSSMessage(int level, int major_status,
 		                   int minor_status,
 		                   const char *message, ...);
 #endif /* HAVE_GSSAPI */
-extern int	cupsdLogJob(cupsd_job_t *job, int level, const char *message, ...)
-#ifdef __GNUC__
-__attribute__ ((__format__ (__printf__, 3, 4)))
-#endif /* __GNUC__ */
-;
+extern int	cupsdLogJob(cupsd_job_t *job, int level, const char *message,
+		            ...) __attribute__((__format__(__printf__, 3, 4)));
 extern int	cupsdLogMessage(int level, const char *message, ...)
-#ifdef __GNUC__
-__attribute__ ((__format__ (__printf__, 2, 3)))
-#endif /* __GNUC__ */
-;
+		__attribute__ ((__format__ (__printf__, 2, 3)));
 extern int	cupsdLogPage(cupsd_job_t *job, const char *page);
 extern int	cupsdLogRequest(cupsd_client_t *con, http_status_t code);
 extern int	cupsdReadConfiguration(void);
@@ -296,5 +298,5 @@ extern int	cupsdWriteErrorLog(int level, const char *message);
 
 
 /*
- * End of "$Id: conf.h 9120 2010-04-23 18:56:34Z mike $".
+ * End of "$Id: conf.h 11221 2013-08-06 16:16:01Z msweet $".
  */
