@@ -1,10 +1,9 @@
 /*
- * "$Id: policy.h 6895 2007-08-30 00:09:27Z mike $"
+ * "$Id: policy.h 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   Policy definitions for the Common UNIX Printing System (CUPS)
- *   scheduler.
+ *   Policy definitions for the CUPS scheduler.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -22,9 +21,15 @@
 typedef struct
 {
   char			*name;		/* Policy name */
-  cups_array_t		*ops;		/* Operations */
+  cups_array_t		*job_access,	/* Private users/groups for jobs */
+			*job_attrs,	/* Private attributes for jobs */
+			*sub_access,	/* Private users/groups for subscriptions */
+			*sub_attrs,	/* Private attributes for subscriptions */
+			*ops;		/* Operations */
 } cupsd_policy_t;
 
+typedef struct cupsd_printer_s cupsd_printer_t;
+
 
 /*
  * Globals...
@@ -47,8 +52,12 @@ extern http_status_t	cupsdCheckPolicy(cupsd_policy_t *p, cupsd_client_t *con,
 extern void		cupsdDeleteAllPolicies(void);
 extern cupsd_policy_t	*cupsdFindPolicy(const char *policy);
 extern cupsd_location_t	*cupsdFindPolicyOp(cupsd_policy_t *p, ipp_op_t op);
+extern cups_array_t	*cupsdGetPrivateAttrs(cupsd_policy_t *p,
+			                      cupsd_client_t *con,
+					      cupsd_printer_t *printer,
+			                      const char *owner);
 
 
 /*
- * End of "$Id: policy.h 6895 2007-08-30 00:09:27Z mike $".
+ * End of "$Id: policy.h 11173 2013-07-23 12:31:34Z msweet $".
  */
