@@ -1,9 +1,9 @@
 /*
- * "$Id: util.c 8700 2009-06-05 21:38:52Z mike $"
+ * "$Id: util.c 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   Mini-daemon utility functions for the Common UNIX Printing System (CUPS).
+ *   Mini-daemon utility functions for CUPS.
  *
- *   Copyright 2007-2009 by Apple Inc.
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -36,13 +36,13 @@
 #ifdef __APPLE__
 #  include <libgen.h>
 extern char **environ;
-#endif /* __APPLE__ */ 
+#endif /* __APPLE__ */
 
 
 /*
  * 'cupsdCompareNames()' - Compare two names.
  *
- * This function basically does a strcasecmp() of the two strings,
+ * This function basically does a _cups_strcasecmp() of the two strings,
  * but is also aware of numbers so that "a2" < "a100".
  */
 
@@ -91,7 +91,7 @@ cupsdCompareNames(const char *s,	/* I - First string */
       else if (!isdigit(*s & 255) && isdigit(*t & 255))
         return (-1);
       else if (!isdigit(*s & 255) || !isdigit(*t & 255))
-        continue;     
+        continue;
 
       if (*s < *t)
         diff = -1;
@@ -160,46 +160,17 @@ cupsdCompareNames(const char *s,	/* I - First string */
 cups_array_t *				/* O - CUPS array */
 cupsdCreateStringsArray(const char *s)	/* I - Comma-delimited strings */
 {
-  cups_array_t	*a;			/* CUPS array */
-  const char	*start,			/* Start of string */
-		*end;			/* End of string */
-  char		*ptr;			/* New string */
-
-
-  if (!s)
+  if (!s || !*s)
     return (NULL);
-
-  if ((a = cupsArrayNew((cups_array_func_t)strcmp, NULL)) != NULL)
-  {
-    for (start = end = s; *end; start = end + 1)
-    {
-     /*
-      * Find the end of the current delimited string...
-      */
-
-      if ((end = strchr(start, ',')) == NULL)
-        end = start + strlen(start);
-
-     /*
-      * Duplicate the string and add it to the array...
-      */
-
-      if ((ptr = calloc(1, end - start + 1)) == NULL)
-        break;
-
-      memcpy(ptr, start, end - start);
-      cupsArrayAdd(a, ptr);
-    }
-  }
-
-  return (a);
+  else
+    return (_cupsArrayNewStrings(s));
 }
 
 
 /*
  * 'cupsdExec()' - Run a program with the correct environment.
  *
- * On Mac OS X, we need to update the CFProcessPath environment variable that
+ * On OS X, we need to update the CFProcessPath environment variable that
  * is passed in the environment so the child can access its bundled resources.
  */
 
@@ -216,7 +187,7 @@ cupsdExec(const char *command,		/* I - Full path to program */
 
 
  /*
-  * Some Mac OS X programs are bundled and need the CFProcessPath environment
+  * Some OS X programs are bundled and need the CFProcessPath environment
   * variable defined.  If the command is a symlink, resolve the link and point
   * to the resolved location, otherwise, use the command path itself.
   */
@@ -496,5 +467,5 @@ cupsdSendIPPTrailer(void)
 
 
 /*
- * End of "$Id: util.c 8700 2009-06-05 21:38:52Z mike $".
+ * End of "$Id: util.c 11173 2013-07-23 12:31:34Z msweet $".
  */
