@@ -1,9 +1,9 @@
 /*
- * "$Id: lpmove.c 7219 2008-01-14 22:00:02Z mike $"
+ * "$Id: lpmove.c 11173 2013-07-23 12:31:34Z msweet $"
  *
- *   "lpmove" command for the Common UNIX Printing System (CUPS).
+ *   "lpmove" command for CUPS.
  *
- *   Copyright 2007 by Apple Inc.
+ *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -22,13 +22,7 @@
  * Include necessary headers...
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <cups/string.h>
-#include <cups/cups.h>
-#include <cups/i18n.h>
-#include <cups/debug.h>
+#include <cups/cups-private.h>
 
 
 /*
@@ -76,7 +70,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 #else
             _cupsLangPrintf(stderr,
-	                    _("%s: Sorry, no encryption support compiled in!\n"),
+	                    _("%s: Sorry, no encryption support."),
 	                    argv[0]);
 #endif /* HAVE_SSL */
 	    break;
@@ -91,7 +85,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        _cupsLangPuts(stderr,
-		              _("Error: need hostname after \'-h\' option!\n"));
+		              _("Error: need hostname after \"-h\" option."));
 		return (1);
               }
 
@@ -100,7 +94,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	    break;
 
 	default :
-	    _cupsLangPrintf(stderr, _("lpmove: Unknown option \'%c\'!\n"),
+	    _cupsLangPrintf(stderr, _("lpmove: Unknown option \"%c\"."),
 	                    argv[i][1]);
 	    return (1);
       }
@@ -122,14 +116,13 @@ main(int  argc,				/* I - Number of command-line arguments */
       dest = argv[i];
     else
     {
-      _cupsLangPrintf(stderr, _("lpmove: Unknown argument \'%s\'!\n"),
-                      argv[i]);
+      _cupsLangPrintf(stderr, _("lpmove: Unknown argument \"%s\"."), argv[i]);
       return (1);
     }
 
   if ((!jobid && !src) || !dest)
   {
-    _cupsLangPuts(stdout, _("Usage: lpmove job/src dest\n"));
+    _cupsLangPuts(stdout, _("Usage: lpmove job/src dest"));
     return (1);
   }
 
@@ -137,8 +130,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (http == NULL)
   {
-    _cupsLangPrintf(stderr,
-		    _("lpmove: Unable to connect to server: %s\n"),
+    _cupsLangPrintf(stderr, _("lpmove: Unable to connect to server: %s"),
 		    strerror(errno));
     return (1);
   }
@@ -208,7 +200,7 @@ move_job(http_t     *http,		/* I - HTTP connection to server */
 
   if (cupsLastError() > IPP_OK_CONFLICT)
   {
-    _cupsLangPrintf(stderr, "lpmove: %s\n", cupsLastErrorString());
+    _cupsLangPrintf(stderr, "lpmove: %s", cupsLastErrorString());
     return (1);
   }
   else
@@ -217,5 +209,5 @@ move_job(http_t     *http,		/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpmove.c 7219 2008-01-14 22:00:02Z mike $".
+ * End of "$Id: lpmove.c 11173 2013-07-23 12:31:34Z msweet $".
  */
