@@ -1,10 +1,10 @@
 #!/bin/sh
 #
-# "$Id: 5.6-lpr.sh 7410 2008-03-29 00:34:23Z mike $"
+# "$Id: 5.6-lpr.sh 7409 2008-03-29 00:26:03Z mike $"
 #
 #   Test the lpr command.
 #
-#   Copyright 2007-2008 by Apple Inc.
+#   Copyright 2007-2012 by Apple Inc.
 #   Copyright 1997-2005 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
@@ -17,7 +17,7 @@
 echo "LPR Default Test"
 echo ""
 echo "    lpr testfile.pdf"
-../berkeley/lpr testfile.pdf 2>&1
+$VALGRIND ../berkeley/lpr testfile.pdf 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -28,8 +28,8 @@ echo ""
 
 echo "LPR Destination Test"
 echo ""
-echo "    lpr -P Test2 testfile.jpg"
-../berkeley/lpr -P Test2 testfile.jpg 2>&1
+echo "    lpr -P Test3 -o fit-to-page testfile.jpg"
+$VALGRIND ../berkeley/lpr -P Test3 -o fit-to-page testfile.jpg 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -41,7 +41,7 @@ echo ""
 echo "LPR Options Test"
 echo ""
 echo "    lpr -P Test1 -o number-up=4 -o job-sheets=standard,none testfile.pdf"
-../berkeley/lpr -P Test1 -o number-up=4 -o job-sheets=standard,none testfile.pdf 2>&1
+$VALGRIND ../berkeley/lpr -P Test1 -o number-up=4 -o job-sheets=standard,none testfile.pdf 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -56,10 +56,16 @@ echo "    lpr -P Test1 testfile.jpg"
 echo "    lpr -P Test2 testfile.jpg"
 i=0
 while test $i -lt $1; do
-	echo "    flood copy $i..." 1>&2
-	../berkeley/lpr -P Test1 testfile.jpg 2>&1 &
-	../berkeley/lpr -P Test2 testfile.jpg 2>&1 &
+	j=1
+	while test $j -le $2; do
+		$VALGRIND ../berkeley/lpr -P test-$j testfile.jpg 2>&1
+		j=`expr $j + 1`
+	done
+
+	$VALGRIND ../berkeley/lpr -P Test1 testfile.jpg 2>&1 &
+	$VALGRIND ../berkeley/lpr -P Test2 testfile.jpg 2>&1 &
 	lprpid=$!
+
 	i=`expr $i + 1`
 done
 wait $lppid
@@ -74,5 +80,5 @@ echo ""
 ./waitjobs.sh
 
 #
-# End of "$Id: 5.6-lpr.sh 7410 2008-03-29 00:34:23Z mike $".
+# End of "$Id: 5.6-lpr.sh 7409 2008-03-29 00:26:03Z mike $".
 #
